@@ -99,7 +99,7 @@ void KCalcBitset::setReadOnly(bool readOnly)
 // Name: setValue
 // Desc: set the value of the bitset based on an unsigned 64-bit number
 //------------------------------------------------------------------------------
-void KCalcBitset::setValue(quint64 value)
+void KCalcBitset::setValue(qint64 value)
 {
     if (value_ == value) {
         // don't waste time if there was no change.
@@ -132,7 +132,7 @@ void KCalcBitset::clear()
 // Name: getValue
 // Desc: returns the bitset value as an unsigned 64-bit number
 //------------------------------------------------------------------------------
-quint64 KCalcBitset::getValue() const
+qint64 KCalcBitset::getValue() const
 {
     return value_;
 }
@@ -162,7 +162,7 @@ void KCalcBitset::slotToggleBit(QAbstractButton *button)
 {
     if (button && !m_readOnly) {
         const int bit = bit_button_group_->id(button);
-        const quint64 nv = getValue() ^ (1LL << bit);
+        const qint64 nv = getValue() ^ (1LL << bit);
         setValue(nv);
         Q_EMIT valueChanged(value_);
     }
