@@ -5,6 +5,7 @@
 
 #pragma once
 
+#include <QKeyEvent>
 #include <QLineEdit>
 #include <QString>
 
@@ -17,19 +18,23 @@ public:
     ~KCalcInputDisplay() override;
 
     void insertToken(const QString &token);
+    void insertTokenNumeric(const QString &token);
     void insertTokenFunction(const QString &token);
 
 public Q_SLOTS:
-    void reset();
-    void slotSetFunctionWrap();
+    void reset(bool clearRedoUndo = false);
+    void setHasResult();
     void slotSetOverwrite();
     void slotSetHardOverwrite();
     void slotClearOverwrite();
-    void slotClearFunctionWrap();
+    void clearHasResult();
+    void undoCalculation();
+    void redoCalculation();
 
 Q_SIGNALS:
 
 protected:
+    void keyPressEvent(QKeyEvent *event) override;
     void focusInEvent(QFocusEvent *e) override;
 
 private Q_SLOTS:
@@ -37,5 +42,5 @@ private Q_SLOTS:
 private:
     bool overwrite_;
     bool hard_overwrite_;
-    bool function_wrap_;
+    bool has_result_;
 };
