@@ -27,6 +27,7 @@ private:
 public:
     enum Type {
         TypeError,
+        TypeComplex,
         TypeInteger,
         TypeFloat,
         TypeFraction,
@@ -38,6 +39,7 @@ public:
     static const KNumber One;
     static const KNumber NegOne;
     static const KNumber OneHundred;
+    static const KNumber I;
     static const KNumber OneThousand;
     static const KNumber PosInfinity;
     static const KNumber NegInfinity;
@@ -62,8 +64,10 @@ public:
 
 #ifdef HAVE_LONG_DOUBLE
     explicit KNumber(long double value);
+    explicit KNumber(long double re, long double img);
 #endif
     explicit KNumber(double value);
+    explicit KNumber(double re, double img);
 
     KNumber(const KNumber &other);
     ~KNumber();
@@ -98,6 +102,8 @@ public:
 
 public:
     KNumber integerPart() const;
+    KNumber realPart() const;
+    KNumber imaginaryPart() const;
 
 public:
     QString toQString(int width = -1, int precision = -1) const;
@@ -137,6 +143,8 @@ public:
     KNumber exp2() const;
     KNumber exp10() const;
     KNumber exp() const;
+    KNumber arg() const;
+    KNumber conj() const;
     KNumber bin(const KNumber &x) const;
 
 public:
